﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Runtime.InteropServices.WindowsRuntime;
using Windows.Devices.Sensors;
using Windows.Foundation;
using Windows.Foundation.Collections;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Controls.Primitives;
using Windows.UI.Xaml.Data;
using Windows.UI.Xaml.Input;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Media.Imaging;
using Windows.UI.Xaml.Navigation;

// The Blank Page item template is documented at http://go.microsoft.com/fwlink/?LinkId=391641

namespace MotionUnlock
{
    /// <summary>
    /// An empty page that can be used on its own or navigated to within a Frame.
    /// </summary>
    public sealed partial class MainPage : Page
    {

        private Accelerometer accelerometer;

        private double dLastX;
        private double dLastY;
        private double dLastZ;

        private double dGravX;
        private double dGravY;
        private double dGravZ;

        private List<LastMovement> mSetPassword;
        private List<LastMovement> mGivPassword;

        private RoughOrientation orientation;
        private LastMovement movement;

        public enum RoughOrientation
        {
            None,
            Up,
            Down,
            Left,
            Right,
            Forward,
            Back,
            UpBack,
            UpLeft,
            UpRight,
            UpForward,
            DownBack,
            DownLeft,
            DownRight,
            DownForward
        };

        public enum LastMovement
        {
            None,
            Up,
            Down,
            Left,
            Right
        };

        public MainPage()
        {
            this.InitializeComponent();
            this.NavigationCacheMode = NavigationCacheMode.Required;

            mSetPassword = new List<LastMovement>();
            mGivPassword = new List<LastMovement>();

            SetPassword();

            this.dGravX = 0;
            this.dGravY = 0;
            this.dGravZ = 0;

            this.accelerometer = Accelerometer.GetDefault();
            this.accelerometer.ReadingChanged += accelerometer_ReadingChanged;
        }

        private RoughOrientation Orientation
        {
            get
            {
                return this.orientation;
            }
            set
            {
                this.orientation = value;
            }
        }


        private LastMovement Movement
        {
            get
            {
                return this.movement;
            }
            set
            {
                this.movement = value;
            }
        }


        async void accelerometer_ReadingChanged(Accelerometer sender, AccelerometerReadingChangedEventArgs args)
        {
            await Dispatcher.RunAsync(Windows.UI.Core.CoreDispatcherPriority.Normal, () =>
            {
                double dCalcX = args.Reading.AccelerationX + dGravX;
                double dCalcY = args.Reading.AccelerationY + dGravY;
                double dCalcZ = args.Reading.AccelerationZ + dGravZ;

                dLastX = args.Reading.AccelerationX;
                dLastY = args.Reading.AccelerationY;
                dLastZ = args.Reading.AccelerationZ;

                // Determine the last distinct motion directiom
                if ((Math.Abs(dCalcX) > Math.Abs(dCalcY)) && (Math.Abs(dCalcX) > Constants.MotionThreshold))
                {
                    if (dCalcX > 0)
                        this.Movement = LastMovement.Right;
                    else
                        this.Movement = LastMovement.Left;
                }
                else if ((Math.Abs(dCalcY) > Math.Abs(dCalcX)) && (Math.Abs(dCalcY) > Constants.MotionThreshold))
                {
                    if (dCalcY > 0)
                        this.Movement = LastMovement.Down;
                    else
                        this.Movement = LastMovement.Up;
                }
                else
                {
                    this.Movement = LastMovement.None;
                }

            });
        }

        /// <summary>
        /// Invoked when this page is about to be displayed in a Frame.
        /// </summary>
        /// <param name="e">Event data that describes how this page was reached.
        /// This parameter is typically used to configure the page.</param>
        protected override void OnNavigatedTo(NavigationEventArgs e)
        {
            // TODO: Prepare page for display here.

            // TODO: If your application contains multiple pages, ensure that you are
            // handling the hardware Back button by registering for the
            // Windows.Phone.UI.Input.HardwareButtons.BackPressed event.
            // If you are using the NavigationHelper provided by some templates,
            // this event is handled for you.
        }

        void Container_Tapped(object sender, TappedRoutedEventArgs e)
        {
            AddMovement();

            if (mGivPassword.Count == mSetPassword.Count)
            {
                for( int iIndex = 0; iIndex < mSetPassword.Count; iIndex++ )
                {
                    if (mSetPassword[iIndex] != mGivPassword[iIndex])
                        return;
                }

                Unlock();
            }
        }

        private void Container_DoubleTapped(object sender, DoubleTappedRoutedEventArgs e)
        {
            // On double tap attempt to zero the accel

            // Determine the angle the phone is being held at
            CalcOrientation(dLastX, dLastY, dLastZ);

            // Calculate the new grav zero values
            dGravX = (-1) * (dLastX);
            dGravY = (-1) * (dLastY);
            dGravZ = (-1) * (dLastZ);
        }

        private RoughOrientation CalcOrientation(double dValX, double dValY, double dValZ)
        {
            RoughOrientation retVal = RoughOrientation.None;

            if (dLastY < -Constants.GravThreshold)
                retVal = RoughOrientation.Down;
            else if (dLastY > Constants.GravThreshold)
                retVal = RoughOrientation.Up;
            else if (dLastX < -Constants.GravThreshold)
                retVal = RoughOrientation.Left;
            else if (dLastX > Constants.GravThreshold)
                retVal = RoughOrientation.Right;
            else if (dLastZ < -Constants.GravThreshold)
                retVal = RoughOrientation.Forward;
            else if (dLastZ > Constants.GravThreshold)
                retVal = RoughOrientation.Back;
            else
                retVal = RoughOrientation.None;

            Orientation = retVal;

            return retVal;
        }

        private void SetPassword()
        {
            mSetPassword.Clear();
            mSetPassword.Add(LastMovement.Left);
            mSetPassword.Add(LastMovement.Right);
            mSetPassword.Add(LastMovement.Left);
            mSetPassword.Add(LastMovement.Up);
        }

        private void AddMovement()
        {
            if (mGivPassword.Count == mSetPassword.Count)
                mGivPassword.RemoveAt(0);      

            mGivPassword.Add(movement);

            PassList.Text = "";
            for( int iIndex = 0; iIndex < mGivPassword.Count; iIndex++ )
            {
               // PassList.Text = PassList.Text + mGivPassword[iIndex].ToString() + "";
            }
        }

        private void Unlock()
        {
            BackgroundImage.Source = new BitmapImage(new Uri("ms-appx:///Assets/UnlockedScreen.png", UriKind.Absolute));
        }
    }
}
